StringUtil = {}

--一个汉字2个字符，英文1个字符
function StringUtil.getLength(str)
    local l = 0
    local lenth = #str
    local i = 1
    while i <= lenth do
        local s = string.byte(str, i, i)
        if s > 0x00 and s < 0x80 then
            l = l + 1
            i = i + 1
        elseif s >= 0xc0 and s < 0xe0 then
            l = l + 2
            i = i + 2
        elseif s >= 0xe0 and s < 0xf0 then
            l = l + 2
            i = i + 3
        elseif s >= 0xf0 and s < 0xf7 then
            l = l + 2
            i = i + 4
        end
    end
    return l
end

local chenesCodes = { "。", "？", "！", "，", "、", "；", "：", "「", "」", "『", "』", "‘", "’", "“", "”", "（", "）", "〔", "〕", "【", "】",
                      "—", "…", "–", "．", "《", "》", "〈", "〉" }

function StringUtil.haveSpecialCode(str)
    local i = 1
    local lenth = #str
    while i <= lenth do
        local s = string.byte(str, i, i)
        if s > 0x40 and s < 0x5b then
            --大写英文
            i = i + 1
        elseif s >= 0x30 and s < 0x3a then
            --数字
            i = i + 1
        elseif s > 0x60 and s < 0x7b then
            --小写英文
            i = i + 1
        elseif s >= 0xc0 and s < 0xe0 then
            i = i + 2
        elseif s >= 0xe0 and s < 0xef then
            for _, ns in ipairs(chenesCodes) do
                if s == string.byte(ns, 1, 1) then
                    return true
                end
            end
            i = i + 3
        else
            return true
        end
    end
    return false
end

function StringUtil.haveChanese(str)
    local lenth = #str
    local i = 1
    while i <= lenth do
        local s = string.byte(str, i, i)
        if s > 0x00 and s < 0x80 then
            i = i + 1
        else
            return true
        end
    end
    return false
end

--奖励类型 1背包道具 2战斗单位 3声望  4金币 5配件 6 荣誉 7 铁矿 8石油 9铅矿 10钛矿 11水晶 12角色经验 14 vip经验 15坦克碎皮 16体力 17将领碎皮
local function getGiftInfoStr(info)
    local funcs = {
        function()
            local item = gItem[info.baseId]
            return StringRes[item.name]
        end,
        function()
            local item = Tank[info.baseId]
            return StringRes[item.name]
        end,
        function()
            return StringRes.prestige
        end,
        function()
            return StringRes.gold
        end,
        function()
            local parts = require("app/data/parts")
            local item = parts[info.baseId]
            return item.name
        end,
        function()
            --6
            return StringRes.honor
        end,
        function()
            return StringRes.ferrum
        end,
        function()
            --8
            return StringRes.oil
        end,
        function()
            --9
            return StringRes.lead
        end,
        function()
            --10
            return StringRes.titanium
        end,
        function()
            --11
            return StringRes.crystal
        end,
        function()
            --12
            return StringRes.exp
        end,
        function()
            return ""
        end,
        function()
            return StringRes.vipExp
        end,
        function()

        end,
        function()
            return StringRes.energy
        end,
        function()
            return gGeneral[info.baseId].generalname
        end
    }
    return string.format("%sx%d", funcs[info.type](), info.num)
end

--奖励类型 1背包道具 2战斗单位 3声望  4金币 5配件 6 荣誉 7 铁矿 8石油 9铅矿 10钛矿 11水晶 12角色经验 14 vip经验 15坦克碎皮 16体力 17将领碎皮
local function getItemInfo(info, defaultType)
    local funcs = {
        function()
            local itemdata = gItem[info.baseId]
            return { icon = itemdata.icon, title = StringRes[itemdata.name], des = StringRes[itemdata.desc], count = info.num }
        end,
        function()
            local itemdata = Tank[info.baseId]
            return { icon = itemdata.icon, title = StringRes[itemdata.name], des = string.format(StringRes[itemdata.dec], itemdata.load), count = info.num,
                     titleColor = Common:getUnitTitleColor(itemdata) }
        end,
        function()
            return { icon = ResIconCfg.prestige, title = StringRes.prestige, des = nil, count = info.num }
        end,
        function()

        end,
        function()
            --5
            local itemdata = require("app/data/parts")[info.baseId]
            local PartsQualityCfg = require("src/app/cfg/PartsQualityCfg")
            local backIcon = (itemdata.type > 4) and PartsQualityCfg[itemdata.que].pieceicon or PartsQualityCfg[itemdata.que].icon
            return { icon = itemdata.icon, title = itemdata.name, des = itemdata.decript, count = info.num
            , backIcon = backIcon, que = itemdata.que }
        end,
        function()
            --6

        end,
        function()

        end,
        function()

        end,
        function()

        end,
        function()
            --10

        end,
        function()

        end,
        function()

        end,
        function()

        end,
        function()

        end,
        function()
            --15

        end,
        function()

        end,
        function()
            local itemdata = gGeneral[info.baseId]
            return { id = info.baseId, icon = itemdata.icon, title = itemdata.generalname, des = itemdata.nation, count = info.num,
                     outIcon = gGeneralQualityIcon[itemdata.que], backIcon = nil, que = itemdata.que }
        end,
    }
    return funcs[info.type or defaultType]()
end

--{baseId=1,type=2,num=3}
function StringUtil.getRewardsData(gifts, defaultType)
    local rewards = {}
    for _, gift in ipairs(gifts) do
        table.insert(rewards, getItemInfo(gift, defaultType))
    end
    return rewards
end

function StringUtil.getRewardString(gifts)
    local re = ""
    for index, info in ipairs(gifts) do
        re = re .. getGiftInfoStr(info)
    end
    return re
end

function StringUtil.encodeURL(s)
    if not s or s == "" then
        return s
    end
    s = string.gsub(s, "([^%w%.%- ])", function(c)
        return string.format("%%%02X", string.byte(c))
    end)
    return string.gsub(s, " ", "+")
end



--奖励类型
--	BAG(1,"背包道具"),
--	WARUNIT(2,"战斗单位"),
--	PRE(3,"声望"),
--	GOLD(4,"金币"),
--	PARTS(5,"配件"),
--	HONOUR(6,"荣誉"),
--	FERRUM(7,"铁矿"),
--	OIL(8,"石油矿"),
--	LEAD(9,"铅矿"),
--	TITANIUM(10,"钛矿"),
--	CRYSTAL(11,"水晶"),
--	EXP(12,"角色经验"),
--	GOLDTIMES(13,"金币倍率"),
--	VIPEXP(14,"VIP经验"),
--	TANK_PIECE(15,"活动坦克碎片"),
--	ENERGY(16,"体力"),
--	COMMANDER(17,"将领碎片"),
--	COMMANDER_MAN(18,"将领");
function StringUtil.getGoodsInfo(goodsId, goodsType)
    if goodsType == 1 then
        printLog("商品id"..goodsId)
        local itemdata = gItem[goodsId]
        return { icon = itemdata.icon, title = StringRes[itemdata.name], des = StringRes[itemdata.desc] }
    elseif goodsType == 2 then
        local itemdata = Tank[goodsId]
        return { icon = itemdata.icon, title = StringRes[itemdata.name], des = string.format(StringRes[itemdata.dec], itemdata.load) }
    elseif goodsType == 3 then
        return { icon = ResIconCfg.prestige }
    elseif goodsType == 4 then
        return { icon = ResIconCfg.gold }
    elseif goodsType == 5 then
        local itemdata = require("app/data/parts")[goodsId]
        return { icon = itemdata.icon, title = itemdata.name, des = itemdata.decript }
    elseif goodsType == 6 then
        return { icon = ResIconCfg.honor }
    elseif goodsType == 7 then
        return { icon = ResIconCfg.ferrum }
    elseif goodsType == 8 then
        return { icon = ResIconCfg.oil }
    elseif goodsType == 9 then
        return { icon = ResIconCfg.lead }
    elseif goodsType == 10 then
        return { icon = ResIconCfg.titanium }
    elseif goodsType == 11 then
        return { icon = ResIconCfg.crystal }
    elseif goodsType == 12 then
        return { icon = ResIconCfg.exp }
    elseif goodsType == 13 then
        return ""
    elseif goodsType == 14 then
        return { icon = ResIconCfg.vipExp }
    elseif goodsType == 15 then
        return ""
    elseif goodsType == 16 then
        return { icon = ResIconCfg.energy }
    elseif goodsType == 17 then
        return ""
    elseif goodsType == 18 then
        local itemdata = gGeneral[goodsId]
        return { icon = itemdata.icon, title = itemdata.generalname, des = itemdata.lifedes }
    end
end